-------------------------------------------------------------------------------
-- ElvUI Equipment Sets Datatext By Lockslap
-------------------------------------------------------------------------------
E, _, V, P, G = unpack(ElvUI)
local DT = E:GetModule("DataTexts")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_EquipmentSetsDT", false)
local EP = LibStub("LibElvUIPlugin-1.0")
local AT = LibStub("AceTimer-3.0")

local join = string.join
local wipe = table.wipe
local sort = table.sort
local pairs = pairs

local displayString = ""
local chatString = ""
local iconString = "|T%s:14:14:0:0:64:64:4:60:4:60|t"
local hexColor = "ffffff"
local lastPanel

-- for drop down menu
local frame = CreateFrame("frame")
local menu = {}

local function GetEquippedSet()
	local num = GetNumEquipmentSets()
	if num == 0 then return false end
	
	for i = 1, num do
		local name, icon, _, isEquipped = GetEquipmentSetInfo(i)
		if isEquipped == true then
			return name, icon
		end
	end
	
	return false
end

local function EquipmentSetClick(button, info, dtFrame)
	UseEquipmentSet(info)
	
	if lastPanel ~= nil then
		local icon = GetEquipmentSetInfoByName(info)
		lastPanel.text:SetFormattedText(displayString, E.db.equipsetsdt.dtIcon == true and iconString:format(("Interface\\Icons\\%s"):format(icon)) or "", info)
	end
end

local function CreateMenu(self, level)
	local num, sets, colorCode = GetNumEquipmentSets(), {}, "|cffffffff"
	wipe(menu)
	
	if num == 0 then return end
	
	for i = 1, num do
		local name, icon, _, isEquipped, _, _, _, missing = GetEquipmentSetInfo(i)
		sets[name] = {
			icon = icon,
			equipped = isEquipped,
			missing = missing,
		}
	end
	sort(sets)
	
	menu.hasArrow = false
	menu.notCheckable = true
	menu.text = L["Choose Equipment Set"]
	menu.isTitle = true
	UIDropDownMenu_AddButton(menu)
	wipe(menu)
	
	for index, value in pairs(sets) do
		if value.equipped then
			colorCode = ("|cff%s"):format(hexColor)
		else
			colorCode = ("|cff%s"):format(value.missing ~= 0 and "ff0000" or "ffffff")
		end
	
		menu.hasArrow = false
		menu.notCheckable = true
		menu.text = index
		menu.colorCode = colorCode
		menu.func = EquipmentSetClick
		menu.arg1 = index
		UIDropDownMenu_AddButton(menu)
		wipe(menu)
	end
end

local function OnClick(self, button)
	if button == "LeftButton" then
		-- wish there was a cleaner way to do this
		ToggleCharacter("PaperDollFrame")
		PaperDollSidebarTab3:Click()
	elseif button == "RightButton" then
		ToggleDropDownMenu(1, nil, frame, self, 0, 0)
	end
end

local function OnEnter(self)
	local num = GetNumEquipmentSets()
	local color = "ffffff"
	local lineString = "%s |cff%s%s|r"
	local itemString = "|cff00ff00%d|r|cffffffff/|r|cff%s%d|r|cffffffff/|r|cff0000ff%d|r|cffffffff/|r|cffff0000%d|r"
	DT:SetupTooltip(self)
	
	if num == 0 then
		DT.tooltip:AddLine(displayString:format("", L["No Equipment Sets"]))
	else
		for i = 1, num do
			local name, icon, _, isEquipped, items, equipped, _, missing, ignored = GetEquipmentSetInfo(i)
			if isEquipped then
				color = hexColor
			else
				color = missing > 0 and "ff0000" or "ffffff"
			end
			DT.tooltip:AddDoubleLine(lineString:format(E.db.equipsetsdt.ttIcon == true and iconString:format(icon) or "", color, name), itemString:format(equipped, hexColor, items, ignored, missing))
		end
	end
	
	if E.db.equipsetsdt.hint == true then
		DT.tooltip:AddLine(" ")
		DT.tooltip:AddDoubleLine(("|cff00ff00%s|r"):format(L["Green Text"]), L["Equipped Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(("|cff%s%s|r"):format(hexColor, L["Color Text"]), L["Total Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(("|cff0000ff%s|r"):format(L["Blue Text"]), L["Ignored Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(("|cffff0000%s|r"):format(L["Red Text"]), L["Missing Items"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(L["Left Click"], L["Toggle Equipment Sets UI"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(L["Right Click"], L["Choose Equipment Set"], 1, 1, 1, 1, 1, 0)
	end
	DT.tooltip:Show()
end

local function OnEvent(self, event)
	local set, icon = GetEquippedSet()
	
	CreateMenu()
	lastPanel = self
	
	if not set then
		self.text:SetFormattedText(displayString, "", L["No Equipped Set"])
		return
	end
	
	self.text:SetFormattedText(displayString, E.db.equipsetsdt.dtIcon == true and iconString:format(icon) or "", set)
end

P["equipsetsdt"] = {
	["dtIcon"] = false,
	["ttIcon"] = true,
	["hint"] = true,
}

local function InjectOptions()
	if not E.Options.args.lockslap then
		E.Options.args.lockslap = {
			type = "group",
			order = -2,
			name = L["Plugins by |cff9382c9Lockslap|r"],
			args = {
				thanks = {
					type = "description",
					order = 1,
					name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
				},
			},
		}
	elseif not E.Options.args.lockslap.args.thanks then
		E.Options.args.lockslap.args.thanks = {
			type = "description",
			order = 1,
			name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
		}
	end
	
	E.Options.args.lockslap.args.equipsetsdt = {
		type = "group",
		name = L["Equipment Sets Datatext"],
		get = function(info) return E.db.equipsetsdt[info[#info]] end,
		set = function(info, value) E.db.equipsetsdt[info[#info]] = value; DT:LoadDataTexts() end,
		args = {
			dtIcon = {
				type = "toggle",
				order = 1,
				name = L["Datatext Icon"],
				desc = L["Display the currently equipped set's icon on the datatext, if available."],
			},
			ttIcon = {
				type = "toggle",
				order = 2,
				name = L["Tooltip Icons"],
				desc = L["Display the icons for the equipment sets in the tooltip, if available."],
			},
			hint = {
				type = "toggle",
				order = 3,
				name = L["Show Hint"],
				desc = L["Show the hint in the tooltip."],
			},
		},
	}
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffffSet:|r %s ", hex, "%s|r")
	chatString = join("", hex, "ElvUI_ESDT|cffffffff: %s|r")
	hexColor = ("%02x%02x%02x"):format(r * 255, g * 255, b * 255) or "ffffff"
	
	if lastPanel ~= nil then
		OnEvent(lastPanel)
	end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

EP:RegisterPlugin(..., InjectOptions)
DT:RegisterDatatext("Equipment Sets", {"PLAYER_ENTERING_WORLD", "WEAR_EQUIPMENT_SET", "EQUIPMENT_SETS_CHANGED"}, OnEvent, nil, OnClick, OnEnter)

frame:SetScript("OnEvent", function(self)
	self.initialize = CreateMenu
	self.displayMode = "MENU"
	self:UnregisterEvent("PLAYER_LOGIN")
end)
frame:RegisterEvent("PLAYER_LOGIN")